// AirMailPickup.uc
// Base class for airmail pickups.
// Most airmail pickups will only need a few elements, see any airmail pickup class for details.

class AirMailPickup extends OwnedPickup
	config(AirMail);

// When airmails disappear they are replaced in MutAirMail.
// This ensures a random distribution of the different powerups for variety.
// The lifetime of a dropped pickup is less than a new pickup. This way if players can't find the pickup
// they're looking for, picking up and dropping airmails will spawn new pickups quicker.

const DSResolution = 0.1;

var float TimeToDestroy;
var float dScale;
var float i;

// If your custom airmail pickup has a limited number of uses or a countdown-style use like the radar, you MUST
// define OurAmountToAdd in defaultproperties. Otherwise the player will only get one.
var config int OurAmountToAdd; 
var config int DESTROY_TIME; // base lifetime of a pickup
var config int DESTROY_VARIATION1; // low end variation
var config int DESTROY_VARIATION2; // high end variation
var config int DROP_DESTROY_TIME; // life of a dropped airmail pickup
var config int DROP_VARIATION1; // low end variation
var config int DROP_VARIATION2; // high end variation

var localized string AlreadyHeldMessage;

// This didn't work, so we don't use it.
simulated function LetsGo()
{
	local vector v;

	FindGround();
	v=vRand()*500;
	if (v.z < 0)
		v.z = -v.z;
	ImpartVelocity(v);
}

// We don't want bots having these, at all
function float BotDesireability( pawn Bot )
{
	return 0;
}

// A little randomness for the pitch
simulated function float GetRandPitch()
{
	return (0.96 + FRand()*0.08);
}

// This didn't work the way I wanted, so it's not used
simulated function ImpartVelocity(vector vel)
{
	bAllowMovement=true;
	SetPhysics(PHYS_Falling);
	Velocity=vel;
}

// Make a smoke puff when you disappear
simulated function MakeAPuff()
{
	spawn(class'SmokeHitPuffGeneric',,,Location,Rotation);
}

// Make a smoke puff when you disappear
simulated event Destroyed()
{
	MakeaPuff();
	Super.Destroyed();
}

state Pickup
{
	// Here we have to check to see if the player already has an air mail pickup.
	// Players may only hold one air mail pickup at a time.
	function Touch(actor Other)
	{
		local inventory Inv, Next;
		local AirMailInv CheckA;
		local bool bAlreadyGotOne;
		local P2Player p2p;

		bAlreadyGotOne = False;
		if (MpPawn(Other) != None)
		{
			// Check their inventory for an airmail pickup.
			inv = MpPawn(Other).Inventory;
			while (Inv != None)
			{
				Next = Inv.Inventory;
				CheckA = AirMailInv(Next);
				If (CheckA != None)
				{
					bAlreadyGotOne = True;
				}
				Inv = Next;
			}
			if (bAlreadyGotOne)
			{
				// They already have one, so tell them and don't let them grab the pickup
				/*p2p = P2Player(MpPawn(Other).Controller);
				if (p2p != None)
					p2p.ClientMessage(AlreadyHeldMessage);*/
				return;
			}
			if (P2Player(MpPawn(Other).Controller) == None) // Don't allow bots to get these, it causes problems
				return;
		}
		Super.Touch(Other);
	}
Begin:
	// checks DropperController (P2PowerupPickup) to see if this was a thrown pickup
	if (DropperController == None)
	{
		TimeToDestroy = DESTROY_TIME + Rand(DESTROY_VARIATION2) + DESTROY_VARIATION1;
		AmountToAdd = OurAmountToAdd;
	}
	else
		TimeToDestroy = DROP_DESTROY_TIME + Rand(DROP_VARIATION2) + DROP_VARIATION1;

	//log (Self @ "will be destroyed in" @ TimeToDestroy @ "seconds... supposedly");
	Sleep(TimeToDestroy);
	//bShrinking=true;
	//GotoState('ShrinkAndDestroy');
	Destroy();
}

// I originally wanted pickups to shrink before they vanish as a visual indicator.
// However it only worked when the player would drop a powerup.
// Instead we just spawn a puff of smoke in Destroyed() since that seems to be the only way I can execuse any
// code on the client side for this actor...
simulated state ShrinkAndDestroy extends Pickup
{
Begin:
	//log(Self@"hit State ShrinkAndDestroy!");
	dScale=DrawScale;
	for (i=dScale; i>0; i-=DSResolution)
	{
		SetDrawScale(i);
		Sleep(DSResolution);
	}
	Destroy();
}

// For some reason the pickups weren't disappearing when picked up but were regenerating
// So destroy them here instead
state Sleeping
{
Begin:
	//log (Self @ "was picked up, so destroying.");
	Destroy();
}

defaultproperties
{
	bAlwaysRelevant=true
	InventoryType=class'AirMailInv'
	DrawType=DT_StaticMesh
	StaticMesh=StaticMesh'Stuff.stuff1.Package'
	DESTROY_TIME = 60; // base lifetime of a pickup
	DESTROY_VARIATION1 = -30; // low end variation
	DESTROY_VARIATION2 = 30; // high end variation
	DROP_DESTROY_TIME = 30; // life of a dropped airmail pickup
	DROP_VARIATION1 = -15; // low end variation
	DROP_VARIATION2 = 15; // high end variation
	PickupMessage="You picked up an Air Mail powerup."
	AlreadyHeldMessage="You can only have one of these at a time."
	OurAmountToAdd=1
}
