// AirMailRules.uc
// GameRules Modifer class for AirMail.

class AirMailRules extends GameRules
	config(AirMail);

// These two allow you to tweak the amount of damage done by regular damage and by vengeance holders
var config float GlobalDamageMult;
var config float VengeanceDamageMult;

// VSArray keeps track of who's been dealing what damage to vengeance holders.
// We couldn't return damage immediately in NetDamage because it caused a bunch of crashes, so we do it on the next tick.
struct VengeanceStruct
{
	var int Damage;
	var pawn Attacker;
	var pawn Other;
	var vector HitLocation;
	var vector Momentum;
};

var array<VengeanceStruct> VSArray;

// If they exploded using the Jihad powerup, add one to their score so they don't lose a point for suiciding.
// May need fixing?
function ScoreKill(Controller Killer, Controller Killed)
{
	if (Killer == Killed && P2Pawn(Killed.Pawn).DyingDamageType == class'amJihadDamage')
		Killer.PlayerReplicationInfo.Score+= 1;

	Super.ScoreKill(Killer, Killed);
}

// Here we check for vengeance, weight gain, zinthar, and the like
function int NetDamage( int OriginalDamage, int Damage, pawn injured, pawn instigatedBy, vector HitLocation, vector Momentum, class<DamageType> DamageType )
{
	local MpPawn Attacker;
	local MpPawn Other;

	Other=MpPawn(Injured);
	Attacker=MpPawn(InstigatedBy);

	// Vengeance is checked by checking the head's DrawScale3d, because it would take too long to go through
	// the guy's inventory looking for vengeance invs.
	if (Other != Attacker && Other != None && DamageType != class'amVengeanceDamage')
		if (Other.MyHead.DrawScale3d != Head(Other.MyHead).RealScale)
		{
			VSArray.Insert(0,1);
			// Apply the vengeance damage mult. This value can be used to tweak the power of vengeance
			VSArray[0].Damage = Int(Damage*VengeanceDamageMult);
			VSArray[0].Attacker = injured;
			VSArray[0].Other = instigatedBy;
			VSArray[0].HitLocation = instigatedBy.Location;
			VSArray[0].Momentum = Momentum;
		}

	// DamageMult exists in FPSPawn, but it doesn't seem to have any effect.
	// The default damagemult is 2.40 for some unknown reason, so we compare everything to 2.40 here.
	// Weight Gain and Zinthar works by altering the DamageMult.
	//log("Attacker.DamageMult is" @ Attacker.DamageMult @ " Original damage was" @ Damage);
	if (Attacker.DamageMult != 2.40 && Attacker != None)
	{
		Damage = int(Damage*Attacker.DamageMult/2.40);
		//log("Using new damage value of" @ Damage);
		//return Damage;
	}

	// Now use the global damage mult
	Damage = int(Damage*GlobalDamageMult);

	//log("Using original damage value");
	// Now see if any other game rules want to mess with our damage
	if ( NextGameRules != None )
		return NextGameRules.NetDamage( OriginalDamage,Damage,injured,instigatedBy,HitLocation,Momentum,DamageType );
	return Damage;
}

// Here we look to see if we need to apply any vengeance damage.
// We can't apply vengeance damage above in NetDamage because it causes problems, so we do it on the next tick.
event Tick(float DeltaTime)
{
	local int i;
	If (VSArray.Length>0)
	{
		for (i=0;i<VSArray.Length;i++)
			if (VSArray[i].Other != None)
				VSArray[i].Other.TakeDamage(VSArray[i].Damage, VSArray[i].Attacker, VSArray[i].HitLocation, VSArray[i].Momentum, class'amVengeanceDamage');

		VSArray.Remove(0,VSArray.Length);
	}
}

defaultproperties
{
	GlobalDamageMult=1.0
	VengeanceDamageMult=1.0
}