// amDogController
// allows us to have the dogs cause amDogBiteDamage that originates from the actual player and not the dog

///////////////////////////////////////////////////////////////////////////////
// DogController
// Copyright 2002 RWS, Inc.  All Rights Reserved.
//
//	Dogs can gain heroes, if that person throws things for them to bring back.
//	While the dog has a hero, he has double the default healthmax.
//
///////////////////////////////////////////////////////////////////////////////
class amDogController extends DogController
	config(AirMail);

// Need a new controller so that dogs can hurt players with the new amDogBiteDamage
// damagetype. Also want it so that the dog's hero is the source of the damage, not
// the dog itself, for scoring and name purposes.

var config float OurPounceDamage;

///////////////////////////////////////////////////////////////////////////////
// consts
///////////////////////////////////////////////////////////////////////////////
const BARK_FREQ  = 0.7;
const GROWL_FREQ = 0.6;

const SNIFF_FREQ    = 0.4;
const PISS_FREQ    = 0.35;
const SIT_LOOP_FREQ = 0.15;
const STAND_LOOP_FREQ = 0.1;
const WALK_AROUND_FREQ = 0.6;
const STOP_ATTACKING_FREQ	=	0.25;

const WHIMPER_AND_RUN_FREQ	=	0.0; // was 0.25

const MIN_VELOCITY_FOR_REAL_MOVEMENT = 30;
const LEG_MOTION_CAUGHT_MAX=1;

const POUNCE_DIST	   = 200;
const POUNCE_DEST_DIST = 400;
const RETREAT_DIST	   = 300;
const ATTACK_RADIUS	   = 150;
const HIT_MOMENTUM	   = 50000;

const GO_AFTER_CAT_FREQ=	0.9; // was 0.75

const HANG_AROUND_HERO	=	450;
const CATCH_AGAIN		=	400;

const ATTACK_DEAD		=	0.3;

const PAWN_END_RADIUS	=	140;

const DISPERSE_RADIUS	=	250;

const HERO_END_RADIUS	=	400;

const LIMP_IF_HURT_FREQ	=	0.2; // was 0.4

const BORED_OF_THROW_MAX=	3;

const MAKE_THROWER_HERO_COUNT=	3;

const MOUTH_BONE	=	'DUMMY07';

const HERO_LOVE_MAX		=	130;
const HERO_LOVE_INC_FOOD=	15;		// love goes up by this much when he feeds me
const HERO_LOVE_INC_PLAY=	5;		// love goes up by this much when plays throw with me once
const HERO_LOVE_START_TIME= 999;		// Time you love hero at first without changing anything. Was 30
const HERO_LOVE_SUB_TIME=	10;		// Subsequent time checks on love of hero
const HERO_LOVE_WALK	=	8;
const HERO_LOVE_TROT	=   20;
const ATTACKER_START_LOVE=	-30;
const PLAYER_DOG_DAMAGE_REDUCTION = 0.35;	// How much less damage our dog takes
const DOG_DAMAGE_DIFF_RATIO		  =	0.02;	// Factor for each game difficulty change average = 5 to 12?
											// that we lower the damage he takes
const MIN_DAMAGE_REDUCTION		  = 0.1;

///////////////////////////////////////////////////////////////////////////
// When attacked
///////////////////////////////////////////////////////////////////////////
function damageAttitudeTo(pawn Other, float Damage)
{
	local TeamInfo HeroTeam, OtherTeam;

	if (Other.PlayerReplicationInfo != None && Hero.PlayerReplicationInfo != None && Level.Game.bTeamGame)
	{
		HeroTeam = Hero.PlayerReplicationInfo.Team;
		OtherTeam = Other.PlayerReplicationInfo.Team;
		if (HeroTeam == OtherTeam)
			GotoStateSave('RunAway');
	}
	else
		Super.DamageAttitudeTo(Other, Damage);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Stand and look around
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Standing
{
Begin:
	MyPawn.PlayAnimStanding();
	MyPawn.SetAnimAction('stand');

	statecount=0;
	Goto('Waiting');

LookingAround:
	if(FRand() <= 0.5)
		MyPawn.PlayContentSound();

	// sometimes sniff some
	if(FRand() <= SNIFF_FREQ)
	{
		MyPawn.PlayInvestigate();
		MyPawn.SetAnimAction('stand');
		statecount=1;
		Goto('Waiting');
	}
	// Very rarely, decide to piss
	if(FRand() <= PISS_FREQ)
	{
		GotoStateSave('Pissing');
	}

CheckToStandAgain:
	// Sometimes stand again
	if(StandAgain())
		Goto('Begin');

	GotoStateSave('Thinking');

Waiting:
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Stand ready to run after hero
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state StandByHero
{
Begin:
	MyPawn.PlayAnimStanding();
	MyPawn.SetAnimAction('stand');

	if(FRand() <= 0.5)
		MyPawn.PlayContentSound();
	statecount=0;
	Goto('Waiting');

LookingAround:

CheckToStandAgain:
	GotoStateSave('Thinking');

Waiting:
	Sleep((Frand()*0.5)+0.1);
	GoToHero(true);
	Goto('Waiting');
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Ready to run by your hero, but you're whining with a hurt leg.. you need
// health
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state LimpByHero
{
Begin:
	Focus = Hero;
	MyPawn.PlayAnimLimping();
	MyPawn.SetAnimAction('limp');

	MyPawn.PlayScaredSound();
	statecount=0;
	Goto('Waiting');

LookingAround:

CheckToStandAgain:
	if(Frand() < LIMP_IF_HURT_FREQ)
	{
		statecount = 0;
		Goto('Begin');
	}
	else
		GotoStateSave('Thinking');

Waiting:
	Sleep((Frand()*0.5)+0.1);
	GoToHero(true);
	Goto('Waiting');
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Barking at interest
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Barking
{
Begin:
	MyPawn.PlayAnimStanding();
	MyPawn.SetAnimAction('stand');

	Sleep(Frand());
	MyPawn.PlayGetScared();
	MyPawn.SetAnimAction('bark');
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Jump on this guy and hurt him
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state PounceOnTarget
{
	///////////////////////////////////////////////////////////////////////////////
	// If touched by normal people run, otherwise, jump in the player's inventory
	///////////////////////////////////////////////////////////////////////////////
	event Touch(actor Other)
	{
		local FPSPawn otherpawn;
		local vector hitpos, dir;

		otherpawn = FPSPawn(Other);

		if(otherpawn != None
			&& otherpawn.Health > 0
			&& otherpawn == Attacker
			&& !bHurtTarget)
		{
			bHurtTarget=true;
			// give it an extra boost
			dir = Normal(Other.Location - MyPawn.Location) + VRand();
			hitpos = Other.Location - Other.CollisionRadius*dir;
			otherpawn.TakeDamage(OurPounceDamage, Hero, hitpos, HIT_MOMENTUM*dir, AttackDamageType);
			MakePeopleScared(class'AnimalAttackMarker');
		}
	}
	function BeginState()
	{
		Super.BeginState();
		// He pounces in this anim
		MyPawn.PlayAttack2();
		MyPawn.SetAnimAction('pounce');
		bHurtTarget=false;
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Run this target down and pounce on them till they're dead
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state AttackTarget
{
	ignores GetReadyToReactToDanger, StartAttacking;

	///////////////////////////////////////////////////////////////////////////////
	// If you Touch the attacker, go into a shredding anim to hurt him
	///////////////////////////////////////////////////////////////////////////////
	event Touch(actor Other)
	{
		local FPSPawn otherpawn;

		otherpawn = FPSPawn(Other);

		if(otherpawn != None
			&& otherpawn.Health > 0
			&& otherpawn == Attacker
			&& (otherpawn != Hero))
		{
			GotoStateSave('ShredAttack');
		}
	}

	///////////////////////////////////////////////////////////////////////////////
	// Check to make sure your attacker is alive.. if not, then go to thinking
	///////////////////////////////////////////////////////////////////////////////
	function BeginState()
	{
		// Don't attack anymore
		if(Attacker == None
			|| Attacker.bDeleteMe)
			GotoStateSave('Thinking');

		if(Attacker.PlayerReplicationInfo.Team == Hero.PlayerReplicationInfo.Team && Level.Game.bTeamGame)
			GotoStateSave('Thinking');

		else if(Attacker.Health <= 0)
		{
			// If the player attacked us, and he's still alive, go right back after him
			if(PlayerAttackedMe != None
				&& PlayerAttackedMe.Health > 0)
				GoAfterPlayerAgain();
			else if(Frand() < STOP_ATTACKING_FREQ)
			{
				// Sometimes piss on the body, sometimes just go back to thinking
				if(FRand() <= PISS_FREQ)
				{
					SetNextState('Pissing');
					SetEndPoint(Attacker.Location, DEFAULT_END_RADIUS);
					SetAttacker(None);
					GotoStateSave('WalkToTarget');
				}
				else
				{
					GotoStateSave('Thinking');
				}
			}
			else
			{
				if(VSize(Attacker.Location - MyPawn.Location) > DEFAULT_END_RADIUS+Attacker.CollisionRadius)
				{
					SetEndPoint(Attacker.Location, DEFAULT_END_RADIUS);
					SetNextState('ShredAttack');
					GotoStateSave('RunToAttacker');
				}
				else
					GotoStateSave('ShredAttack');
			}
		}
	}

Begin:
	MyPawn.PlayAngrySound();
	SetNextState('PounceOnTarget');
	// Make the randomly jump at you, some farther away than others
	SetEndGoal(Attacker, POUNCE_DIST + (2*FRand()*POUNCE_DIST));
	GotoStateSave('RunToPounce');
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Wrestling the thing back and forth, hurting it
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state ShredAttack
{
	///////////////////////////////////////////////////////////////////////////////
	// If touched by normal people run, otherwise, jump in the player's inventory
	///////////////////////////////////////////////////////////////////////////////
	event Touch(actor Other)
	{
		local FPSPawn otherpawn;
		local vector hitpos, dir;

		otherpawn = FPSPawn(Other);

		if(otherpawn != None
//			&& otherpawn.Health > 0
			&& otherpawn == Attacker
			&& !bHurtTarget)
		{
			bHurtTarget=true;
			dir = Normal(Other.Location - MyPawn.Location);
			hitpos = Other.Location - Other.CollisionRadius*dir;
			otherpawn.TakeDamage(OurPounceDamage, Hero, hitpos, HIT_MOMENTUM*VRand(), AttackDamageType);
			MakePeopleScared(class'AnimalAttackMarker');
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
defaultproperties
{
	OurPounceDamage = 40 // we can change pounce damage here
	AttackDamageType = class'amDogBiteDamage'
}
