// AirMailInv.uc
// Parent class for all AirMail Inventorys
// Custom AirMail inventory should extend from this class.

class AirMailInv extends OwnedInv
	config(AirMail);

// This is the parent class for all Air Mail inventory classes.
// It defines the default properties for Air Mail inventory and allows for
// client messages describing the air mail pickup.

var localized string Description; // blurb shown to the player when they pick it up
var localized string AutoUseDesc; // blurb shown to the player if Activate() isn't overridden

var P2MoCapPawn.Sbolton OurBolton; // some powerups use boltons
var int OurBoltonIndex;
var MpPawn OurOwner; // MpPawn version of Owner
var P2Player OurOwnerController; // P2Player version of Owner.Controller

var config float SLEEP_TIME;

replication
{
	// stuff client sends to the server
	reliable if (ROLE < ROLE_Authority)
		AirMailActivate, ServerAirMailDrop;
}

// just a little randomness for the pitch
simulated function float GetRandPitch()
{
	return (0.96 + FRand()*0.08);
}

// Boltons not added unless this function is called by GrantPowers/RevokePowers
function AddBolton(P2MoCapPawn Other)
{
	local int i;
	local int BoltonAdded;

	BoltonAdded=-1;
	for (i=0; i<Other.MAX_BOLTONS; i++)
	{
		if (Other.boltons[i].Part == None) // look for a bolton that isn't being used for something else
		{
			Other.Boltons[i]=OurBolton;
			BoltonAdded=i;
			break;
		}
	}
	if (BoltonAdded > -1) // keep track of which bolton we used
	{
		OurBoltonIndex=BoltonAdded;
		Other.SetupBoltons();
	}
}

function RemoveBolton(MpPawn Other)
{
	if (OurBoltonIndex > -1)
		if (Other.Boltons[OurBoltonIndex].part != None) // if it's still there, destroy it
		{
			Other.Boltons[OurBoltonIndex].part.Destroy();
			Other.Boltons[OurBoltonIndex].part = None;
		}
}

// In your custom AirMail class, use this function to grant powers to the player.
// You can use OurOwner and OurOwnerController here if you like.
function GrantPowers(MPPawn Other); // STUB

// In your custom AirMail class, use this function to revoke any powers that must be revoked when the package is dropped.
// You can NOT refer to OurOwner or OurOwnerController here because they may be invalid.
function RevokePowers(MPPawn Other); // STUB

// tell the player what he got
// serves as info for new players or a reminder for experienced players.
function PickupFunction(Pawn Other)
{
	Super.PickupFunction(Other); // do super first
	If (PlayerController(Other.Controller) != None)
		PlayerController(Other.Controller).ClientMessage(Description); // tell 'em what they got
	else
		Destroy(); // don't let bots have these!!!
	OurOwner = MpPawn(Other); // set up OurOwner
	If (OurOwner != None)
	{
		GrantPowers(OurOwner); // grant powers
		OurOwnerController = P2Player(OurOwner.Controller); // set up OutOwnerController
	}

	SetTimer(SLEEP_TIME, true); // initiate the timer
}

// Dummied out, power revocation handled in Destroyed()
/*
// DropFrom is not used in the default pickup, however it is useful for taking away stuff you've granted
// the player in PickupFunction.
function DropFrom(vector StartLocation)
{
	RevokePowers(OurOwner);

	Super.DropFrom(StartLocation);
}
*/

function Destroyed()
{
	RevokePowers(OurOwner); // remove powers
	Super.Destroyed();
}

// Timer function, fixes the exploit where a player could throw their weapon and get infinite Air Mail powers.
// ALL SUBCLASSES MUST CALL SUPER.TIMER()!!!!
function Timer()
{
	// The controller doesn't know we still exist.
	// If this is the case then just destroy ourselves.
	// Power removal handled in Destroyed()
	if (OurOwnerController.GetInv(InventoryGroup,GroupOffset) == None)
		Destroy();
}

// If your pickup must be activated to be used, DO NOT call the Super version of Activate!
// Instead override Activate with your desired effect.
// If your pickup is automatic, define AutoUseDesc as necessary with special instructions.
function Activate()
{
	local Pawn CheckPawn;
	local PlayerController pc;

	CheckPawn = Pawn(Owner);
	if (CheckPawn != None) {
		pc = PlayerController(CheckPawn.Controller);
		if (pc != None)
			pc.ClientMessage(AutoUseDesc);
	}
}

exec function AirMailActivate()
{
	Activate();
}

exec function AirMailDrop()
{
	ServerAirMailDrop();
}

function ServerAirMailDrop()
{
	OurOwner.TossThisInventory(OurOwnerController.GenTossVel(), self);
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////
defaultproperties
{
	//PickupClass=class'AirMailPickup'
	Icon=Texture'HUDPack.Icons.Icon_Inv_Package'
	InventoryGroup=900
	bDisplayAmount=true
	bMustBeDropped=True
	// if the player leaves we still need this pickup around. It's not necessary but
	// it helps keep the number of pickups in equilibrium.
	Description="AIR MAIL: Air Mail powerup."
	AutoUseDesc="This powerup is automatic and does not need to be activated."
	SLEEP_TIME=1.000000
	OurBoltonIndex=-1
}
